@@ -3,17 +3,7 @@
 // Lag vectors and lists recursively, in place, with custom order and run lengths
 // Author: Nick Christofides
 
-// Internal fast check for valid order values (here they are 0-indexed)
-// Where rng = length(x) - 1
-void check_order_value(unsigned int x, unsigned int rng, int n_prot){
-  if (!(x <= rng)){
-    Rf_unprotect(n_prot);
-    Rf_error("order must be an integer vector of unique values between 1 and length(x)");
-  }
-}
-
-[[cpp11::register]]
-SEXP cpp_lag(SEXP x, R_xlen_t k, SEXP fill, bool set, bool recursive) {
+SEXP lag(SEXP x, R_xlen_t k, SEXP fill, bool set) {
   R_xlen_t size = Rf_xlength(x);
   R_xlen_t fill_size = Rf_xlength(fill);
   int NP = 0;
@@ -25,7 +15,7 @@ SEXP cpp_lag(SEXP x, R_xlen_t k, SEXP fill, bool set, bool recursive) {
   if (ALTREP(x)){
     set = true;
   }
-  SEXP xvec = Rf_protect(altrep_materialise(x)); ++NP;
+  SEXP xvec = SHIELD(altrep_materialise(x)); ++NP;
   if (set_and_altrep){
     Rf_warning("Cannot lag an ALTREP by reference, a copy has been made.\n\tEnsure the result is assigned to an object if used in further calculations\n\te.g. `x <- lag_(x, set = TRUE)`");
   }
@@ -41,9 +31,8 @@ SEXP cpp_lag(SEXP x, R_xlen_t k, SEXP fill, bool set, bool recursive) {
     if (fill_size >= 1){
       fill_value = Rf_asInteger(fill);
     }
-    out = Rf_protect(set ? xvec : Rf_duplicate(xvec));
-    ++NP;
-    int *p_out = INTEGER(out);
+    out = SHIELD(set ? xvec : cpp_semi_copy(xvec)); ++NP;
+    int* RESTRICT p_out = INTEGER(out);
     int *p_x = INTEGER(xvec);
     if (k >= 0){
       memmove(&p_out[k], &p_x[0], (size - k) * sizeof(int));
@@ -54,44 +43,30 @@ SEXP cpp_lag(SEXP x, R_xlen_t k, SEXP fill, bool set, bool recursive) {
       OMP_FOR_SIMD
       for (R_xlen_t i = size - 1; i >= size + k; --i) p_out[i] = fill_value;
     }
-    if (!Rf_isNull(Rf_getAttrib(xvec, R_NamesSymbol))){
-      SEXP old_names = Rf_protect(Rf_getAttrib(xvec, R_NamesSymbol));
-      ++NP;
-      SEXP new_names = Rf_protect(cpp_lag(old_names, k, R_NilValue, set, recursive));
-      ++NP;
-      Rf_setAttrib(out, R_NamesSymbol, new_names);
-    }
     break;
   }
   case CHEAPR_INT64SXP: {
     k = k >= 0 ? std::min(size, k) : std::max(-size, k);
-    long long int fill_value = NA_INTEGER64;
+    int_fast64_t fill_value = NA_INTEGER64;
     if (fill_size >= 1){
-      SEXP temp_fill = Rf_protect(coerce_vector(fill, CHEAPR_INT64SXP)); ++NP;
+      SEXP temp_fill = SHIELD(coerce_vector(fill, CHEAPR_INT64SXP)); ++NP;
       fill_value = INTEGER64_PTR(temp_fill)[0];
     }
-    out = Rf_protect(set ? xvec : Rf_duplicate(xvec)); ++NP;
-    long long int *p_out = INTEGER64_PTR(out);
-    long long int *p_x = INTEGER64_PTR(xvec);
+    out = SHIELD(set ? xvec : cpp_semi_copy(xvec)); ++NP;
+    int_fast64_t* RESTRICT p_out = INTEGER64_PTR(out);
+    int_fast64_t *p_x = INTEGER64_PTR(xvec);
 
-    // sizeof(long long int) == sizeof(double), both are 8 bytes
+    // sizeof(int_fast64_t) == sizeof(double), both are 8 bytes
 
     if (k >= 0){
-      memmove(&p_out[k], &p_x[0], (size - k) * sizeof(long long int));
+      memmove(&p_out[k], &p_x[0], (size - k) * sizeof(int_fast64_t));
       OMP_FOR_SIMD
       for (R_xlen_t i = 0; i < k; ++i) p_out[i] = fill_value;
     } else {
-      memmove(&p_out[0], &p_x[-k], (size + k) * sizeof(long long int));
+      memmove(&p_out[0], &p_x[-k], (size + k) * sizeof(int_fast64_t));
       OMP_FOR_SIMD
       for (R_xlen_t i = size - 1; i >= size + k; --i) p_out[i] = fill_value;
     }
-    if (!Rf_isNull(Rf_getAttrib(xvec, R_NamesSymbol))){
-      SEXP old_names = Rf_protect(Rf_getAttrib(xvec, R_NamesSymbol));
-      ++NP;
-      SEXP new_names = Rf_protect(cpp_lag(old_names, k, R_NilValue, set, recursive));
-      ++NP;
-      Rf_setAttrib(out, R_NamesSymbol, new_names);
-    }
     break;
   }
   case REALSXP: {
@@ -100,8 +75,8 @@ SEXP cpp_lag(SEXP x, R_xlen_t k, SEXP fill, bool set, bool recursive) {
     if (fill_size >= 1){
       fill_value = Rf_asReal(fill);
     }
-    out = Rf_protect(set ? xvec : Rf_duplicate(xvec)); ++NP;
-    double *p_out = REAL(out);
+    out = SHIELD(set ? xvec : cpp_semi_copy(xvec)); ++NP;
+    double* RESTRICT p_out = REAL(out);
     double *p_x = REAL(xvec);
     if (k >= 0){
       memmove(&p_out[k], &p_x[0], (size - k) * sizeof(double));
@@ -112,23 +87,16 @@ SEXP cpp_lag(SEXP x, R_xlen_t k, SEXP fill, bool set, bool recursive) {
       OMP_FOR_SIMD
       for (R_xlen_t i = size - 1; i >= size + k; --i) p_out[i] = fill_value;
     }
-    if (!Rf_isNull(Rf_getAttrib(xvec, R_NamesSymbol))){
-      SEXP old_names = Rf_protect(Rf_getAttrib(xvec, R_NamesSymbol));
-      ++NP;
-      SEXP new_names = Rf_protect(cpp_lag(old_names, k, R_NilValue, set, recursive));
-      ++NP;
-      Rf_setAttrib(out, R_NamesSymbol, new_names);
-    }
     break;
   }
   case CPLXSXP: {
     k = k >= 0 ? std::min(size, k) : std::max(-size, k);
-    SEXP fill_sexp = Rf_protect(Rf_allocVector(CPLXSXP, 1)); ++NP;
+    SEXP fill_sexp = SHIELD(new_vec(CPLXSXP, 1)); ++NP;
     Rcomplex *p_fill = COMPLEX(fill_sexp);
     p_fill[0].i = NA_REAL;
     p_fill[0].r = NA_REAL;
     Rcomplex fill_value = fill_size >= 1 ? Rf_asComplex(fill) : COMPLEX(fill_sexp)[0];
-    out = Rf_protect(set ? xvec : Rf_duplicate(xvec)); ++NP;
+    out = SHIELD(set ? xvec : cpp_semi_copy(xvec)); ++NP;
     Rcomplex *p_out = COMPLEX(out);
     Rcomplex *p_x = COMPLEX(xvec);
     if (k >= 0){
@@ -139,30 +107,23 @@ SEXP cpp_lag(SEXP x, R_xlen_t k, SEXP fill, bool set, bool recursive) {
       OMP_FOR_SIMD
       for (R_xlen_t i = size - 1; i >= size + k; --i) SET_COMPLEX_ELT(out, i, fill_value);
     }
-    if (!Rf_isNull(Rf_getAttrib(xvec, R_NamesSymbol))){
-      SEXP old_names = Rf_protect(Rf_getAttrib(xvec, R_NamesSymbol));
-      ++NP;
-      SEXP new_names = Rf_protect(cpp_lag(old_names, k, R_NilValue, set, recursive));
-      ++NP;
-      Rf_setAttrib(out, R_NamesSymbol, new_names);
-    }
     break;
   }
   case STRSXP: {
     k = k >= 0 ? std::min(size, k) : std::max(-size, k);
-    SEXP fill_char = Rf_protect(fill_size >= 1 ? Rf_asChar(fill) : NA_STRING);
+    SEXP fill_char = SHIELD(fill_size >= 1 ? Rf_asChar(fill) : NA_STRING);
     ++NP;
-    out = Rf_protect(set ? xvec : Rf_duplicate(xvec)); ++NP;
+    out = SHIELD(set ? xvec : cpp_semi_copy(xvec)); ++NP;
     const SEXP *p_out = STRING_PTR_RO(out);
     if (set){
       R_xlen_t tempi;
       // If k = 0 then no lag occurs
       if (std::abs(k) >= 1){
-        SEXP lag_temp = Rf_protect(Rf_allocVector(STRSXP, std::abs(k)));
+        SEXP lag_temp = SHIELD(new_vec(STRSXP, std::abs(k)));
         ++NP;
-        SEXP tempv = Rf_protect(Rf_allocVector(STRSXP, 1));
+        SEXP tempv = SHIELD(new_vec(STRSXP, 1));
         ++NP;
-        const SEXP* __restrict__ p_lag = STRING_PTR_RO(lag_temp);
+        const SEXP *p_lag = STRING_PTR_RO(lag_temp);
         // Positive lags
         if (k >= 0){
           for (R_xlen_t i = 0; i < k; ++i) {
@@ -199,28 +160,21 @@ SEXP cpp_lag(SEXP x, R_xlen_t k, SEXP fill, bool set, bool recursive) {
         for (R_xlen_t i = size + k - 1; i >= 0; --i) SET_STRING_ELT(out, i, p_x[i - k]);
       }
     }
-    if (!Rf_isNull(Rf_getAttrib(xvec, R_NamesSymbol))){
-      SEXP old_names = Rf_protect(Rf_getAttrib(xvec, R_NamesSymbol));
-      ++NP;
-      SEXP new_names = Rf_protect(cpp_lag(old_names, k, R_NilValue, set, recursive));
-      ++NP;
-      Rf_setAttrib(out, R_NamesSymbol, new_names);
-    }
     break;
   }
   case RAWSXP: {
     k = k >= 0 ? std::min(size, k) : std::max(-size, k);
-    SEXP raw_sexp = Rf_protect(Rf_coerceVector(fill, RAWSXP));
+    SEXP raw_sexp = SHIELD(coerce_vec(fill, RAWSXP));
     Rbyte fill_raw = fill_size == 0 ? RAW(Rf_ScalarRaw(0))[0] : RAW(raw_sexp)[0];
     ++NP;
-    out = Rf_protect(set ? xvec : Rf_duplicate(xvec));
+    out = SHIELD(set ? xvec : cpp_semi_copy(xvec));
     ++NP;
     Rbyte *p_out = RAW(out);
     if (set){
       R_xlen_t tempi;
       // If k = 0 then no lag occurs
       if (std::abs(k) >= 1){
-        SEXP lag_temp = Rf_protect(Rf_allocVector(RAWSXP, std::abs(k)));
+        SEXP lag_temp = SHIELD(new_vec(RAWSXP, std::abs(k)));
         ++NP;
         Rbyte *p_lag = RAW(lag_temp);
         // Positive lags
@@ -261,40 +215,20 @@ SEXP cpp_lag(SEXP x, R_xlen_t k, SEXP fill, bool set, bool recursive) {
         }
       }
     }
-    if (!Rf_isNull(Rf_getAttrib(xvec, R_NamesSymbol))){
-      SEXP old_names = Rf_protect(Rf_getAttrib(xvec, R_NamesSymbol));
-      ++NP;
-      SEXP new_names = Rf_protect(cpp_lag(old_names, k, R_NilValue, set, recursive));
-      ++NP;
-      Rf_setAttrib(out, R_NamesSymbol, new_names);
-    }
     break;
   }
   case VECSXP: {
-    if (recursive){
-    const SEXP *p_x = VECTOR_PTR_RO(xvec);
-    out = Rf_protect(Rf_allocVector(VECSXP, size));
-    ++NP;
-    SHALLOW_DUPLICATE_ATTRIB(out, xvec);
-    for (R_xlen_t i = 0; i < size; ++i){
-      SET_VECTOR_ELT(out, i, cpp_lag(p_x[i], k, fill, set, true));
-    }
-  } else {
     k = k >= 0 ? std::min(size, k) : std::max(-size, k);
-    SEXP fill_value = Rf_protect(Rf_coerceVector(fill_size >= 1 ? fill : R_NilValue, VECSXP));
-    ++NP;
-    out = Rf_protect(set ? xvec : Rf_allocVector(VECSXP, size));
-    ++NP;
+    SEXP fill_value = SHIELD(coerce_vec(fill_size >= 1 ? fill : R_NilValue, VECSXP)); ++NP;
+    out = SHIELD(set ? xvec : new_vec(VECSXP, size)); ++NP;
     const SEXP *p_out = VECTOR_PTR_RO(out);
     if (set){
       R_xlen_t tempi;
       // If k = 0 then no lag occurs
       if (std::abs(k) >= 1){
-        SEXP lag_temp = Rf_protect(Rf_allocVector(VECSXP, std::abs(k)));
-        ++NP;
-        SEXP tempv = Rf_protect(Rf_allocVector(VECSXP, 1));
-        ++NP;
-        const SEXP* __restrict__ p_lag = VECTOR_PTR_RO(lag_temp);
+        SEXP lag_temp = SHIELD(new_vec(VECSXP, std::abs(k))); ++NP;
+        SEXP tempv = SHIELD(new_vec(VECSXP, 1)); ++NP;
+        const SEXP *p_lag = VECTOR_PTR_RO(lag_temp);
         // Positive lags
         if (k >= 0){
           for (R_xlen_t i = 0; i < k; ++i) {
@@ -333,27 +267,40 @@ SEXP cpp_lag(SEXP x, R_xlen_t k, SEXP fill, bool set, bool recursive) {
         }
       }
     }
-    if (!Rf_isNull(Rf_getAttrib(xvec, R_NamesSymbol))){
-      SEXP old_names = Rf_protect(Rf_getAttrib(xvec, R_NamesSymbol));
-      ++NP;
-      SEXP new_names = Rf_protect(cpp_lag(old_names, k, R_NilValue, set, recursive));
-      ++NP;
-      Rf_setAttrib(out, R_NamesSymbol, new_names);
-    }
-  }
-  break;
+    break;
   }
   default: {
-    Rf_unprotect(NP);
+    YIELD(NP);
     Rf_error("%s cannot handle an object of type %s", __func__, Rf_type2char(TYPEOF(xvec)));
   }
   }
-  Rf_unprotect(NP);
+  YIELD(NP);
   return out;
 }
 
+// The reason for having a separate function for doing the recursion is so that
+// __restrict__ pointers can be more safely used
+
 [[cpp11::register]]
-SEXP cpp_lag2(SEXP x, SEXP lag, SEXP order, SEXP run_lengths, SEXP fill, bool recursive){
+SEXP cpp_lag(SEXP x, R_xlen_t k, SEXP fill, bool set, bool recursive){
+  SEXP out = R_NilValue;
+  if (recursive && TYPEOF(x) == VECSXP){
+    R_xlen_t size = Rf_xlength(x);
+    const SEXP *p_x = VECTOR_PTR_RO(x);
+    out = SHIELD(new_vec(VECSXP, size));
+    SHALLOW_DUPLICATE_ATTRIB(out, x);
+    for (R_xlen_t i = 0; i < size; ++i){
+      SET_VECTOR_ELT(out, i, cpp_lag(p_x[i], k, fill, set && !ALTREP(p_x[i]), true));
+    }
+  } else {
+    out = SHIELD(lag(x, k, fill, set));
+    set_names(out, lag(get_names(x), k, fill, set && !ALTREP(x)));
+  }
+  YIELD(1);
+  return out;
+}
+
+SEXP lag2(SEXP x, SEXP lag, SEXP order, SEXP run_lengths, SEXP fill){
   int o_size = Rf_length(order);
   int rl_size = Rf_length(run_lengths);
   int lag_size = Rf_length(lag);
@@ -365,9 +312,8 @@ SEXP cpp_lag2(SEXP x, SEXP lag, SEXP order, SEXP run_lengths, SEXP fill, bool re
   if (lag_size < 1){
     Rf_error("lag must be a non-zero length integer vector");
   }
-  bool has_order = !Rf_isNull(order);
-  bool has_rl = !Rf_isNull(run_lengths);
-  bool recycle_lag = lag_size != 1;
+  bool has_order = !is_null(order);
+  bool has_rl = !is_null(run_lengths);
 
   // When order is NULL we run through x from left to right (as usual)
   // When run_lengths is NULL we run through x without resetting
@@ -378,47 +324,26 @@ SEXP cpp_lag2(SEXP x, SEXP lag, SEXP order, SEXP run_lengths, SEXP fill, bool re
   // hence order/run_lengths should remain NULL throughout each recursion
   // if they are NULL
 
-  SEXP dummy_vec1 = Rf_protect(Rf_allocVector(INTSXP, 0));
-  ++NP;
-  SEXP dummy_vec2 = Rf_protect(Rf_allocVector(INTSXP, 0));
-  ++NP;
-  Rf_protect(lag = Rf_coerceVector(lag, INTSXP));
-  ++NP;
-  Rf_protect(order = has_order ? Rf_coerceVector(order, INTSXP) : R_NilValue);
-  ++NP;
-  Rf_protect(run_lengths = has_rl ? Rf_coerceVector(run_lengths, INTSXP) : R_NilValue);
-  ++NP;
-  // std::variant<int*, double*> p_o;
-  // typedef std::conditional<true, int*, double*>::type p_o;
-  int *p_o = INTEGER(has_order ? order : dummy_vec1);
-  int *p_rl = INTEGER(has_rl ? run_lengths : dummy_vec2);
-  int *p_lag = INTEGER(lag);
+  SHIELD(lag = coerce_vec(lag, INTSXP)); ++NP;
+  SHIELD(order = has_order ? coerce_vec(order, INTSXP) : R_NilValue); ++NP;
+  SHIELD(run_lengths = has_rl ? coerce_vec(run_lengths, INTSXP) : R_NilValue); ++NP;
+  int foo = 42;
+  const int *p_o = &foo;
+  const int *p_rl = &foo;
+  if (has_order) p_o = INTEGER(order);
+  if (has_rl) p_rl = INTEGER(run_lengths);
+  const int *p_lag = INTEGER(lag);
   int rl; // Run-length
   int run_start = 0; // Start index of current run
   int run_end = 0; // End index of current run
   int oi; // Indices (specified by order vector) to lag
   int k; // Lag
-  int lag1 = p_lag[0];
   // Manually set run rl size to 1 if run_lengths = NULL (checked prior)
   if (!has_rl) rl_size = 1;
-
-  // Macro to lag names(x)
-#define CHEAPR_LAG_R_NAMES                                                                                  \
-  if (!Rf_isNull(Rf_getAttrib(x, R_NamesSymbol))){                                                          \
-    SEXP old_names = Rf_protect(Rf_getAttrib(x, R_NamesSymbol));                                            \
-    ++NP;                                                                                                   \
-    SEXP new_names = Rf_protect(cpp_lag2(old_names, lag, order, run_lengths, R_NilValue, recursive));       \
-    ++NP;                                                                                                   \
-    Rf_setAttrib(out, R_NamesSymbol, new_names);                                                            \
-  }                                                                                                         \
-
-// Initialise range of possible order values to fast check
-// user-supplied order values
-unsigned int o_rng = o_size - 1;
-  SEXP out;
+  bool recycle_lag = lag_size != 1;
+  SEXP out = R_NilValue;
   switch(CHEAPR_TYPEOF(x)){
   case NILSXP: {
-    out = R_NilValue;
     break;
   }
   case LGLSXP:
@@ -427,13 +352,13 @@ unsigned int o_rng = o_size - 1;
     if (has_order && (size != o_size)){
       Rf_error("length(order) must equal length(x) (%d)", size);
     }
-    int *p_x = INTEGER(x);
+    const int *p_x = INTEGER(x);
     int fill_value = NA_INTEGER;
     if (fill_size >= 1){
       fill_value = Rf_asInteger(fill);
     }
-    out = Rf_protect(Rf_duplicate(x)); ++NP;
-    int *p_out = INTEGER(out);
+    out = SHIELD(cpp_semi_copy(x)); ++NP;
+    int* RESTRICT p_out = INTEGER(out);
     for (int i = 0; i != rl_size; ++i){
       run_start = run_end; // Start at the end of the previous run
       // Manually set run rl if order = NULL
@@ -442,43 +367,38 @@ unsigned int o_rng = o_size - 1;
 
       // If any run-lengths are negative (or NA) we stop
       if (rl < 0){
-        Rf_unprotect(NP);
+        YIELD(NP);
         Rf_error("All run lengths must be non-NA and >= 0");
       }
 
       // If the cumulative run-length exceeds length(x) we stop
       if (run_end > size){
-        Rf_unprotect(NP);
+        YIELD(NP);
         Rf_error("sum(run_lengths) must be equal to length(x) (%d)", size);
       }
-      // int trick to calculate inclusive bounded between(x, lo, hi)
-      // unsigned int rng = (run_end - 1) - run_start;
       // Loop starting from the end of the previous run-length
       for (int j = run_start; j != run_end; ++j){
         // Check that order value is valid
         if (has_order){
           oi = p_o[j] - 1;
-          check_order_value(oi, o_rng, NP);
+          if (oi < 0 || oi >= size){
+            Rf_error("`order` must be an integer vector of unique values between 1 and `length(x)`");
+          }
         } else {
           oi = j;
         }
-        // Costly to use % if we don't need to
-        // k = recycle_lag ? p_lag[j % lag_size] : lag1;
-        k = recycle_lag ? p_lag[oi % lag_size] : lag1;
+        k = p_lag[recycle_lag ? oi % lag_size : 0];
         if (k >= 0){
           p_out[oi] = (j - run_start) >= k ? p_x[has_order ? p_o[j - k] - 1 : j - k] : fill_value;
         } else {
           p_out[oi] = (j - run_end) < k ? p_x[has_order ? p_o[j - k] - 1 : j - k] : fill_value;
         }
-        // lagi = (k != NA_INTEGER) ? j - k : rng + 1;
-        // p_out[oi] = unsigned(lagi - run_start) <= rng ? p_x[has_order ? p_o[lagi] - 1 : lagi] : fill_value;
       }
     }
     if (run_end != size){
-      Rf_unprotect(NP);
+      YIELD(NP);
       Rf_error("sum(run_lengths) must be equal to length(x) (%d)", size);
     }
-    CHEAPR_LAG_R_NAMES;
     break;
   }
   case CHEAPR_INT64SXP: {
@@ -486,15 +406,14 @@ unsigned int o_rng = o_size - 1;
     if (has_order && (size != o_size)){
       Rf_error("length(order) must equal length(x) (%d)", size);
     }
-    long long int *p_x = INTEGER64_PTR(x);
-    long long int fill_value = NA_INTEGER64;
+    const int_fast64_t *p_x = INTEGER64_PTR(x);
+    int_fast64_t fill_value = NA_INTEGER64;
     if (fill_size >= 1){
-      SEXP temp_fill = Rf_protect(coerce_vector(fill, CHEAPR_INT64SXP)); ++NP;
+      SEXP temp_fill = SHIELD(coerce_vector(fill, CHEAPR_INT64SXP)); ++NP;
       fill_value = INTEGER64_PTR(temp_fill)[0];
     }
-    out = Rf_protect(Rf_duplicate(x));
-    ++NP;
-    long long int *p_out = INTEGER64_PTR(out);
+    out = SHIELD(cpp_semi_copy(x)); ++NP;
+    int_fast64_t* RESTRICT p_out = INTEGER64_PTR(out);
     for (int i = 0; i != rl_size; ++i){
       run_start = run_end; // Start at the end of the previous run
       rl = has_rl ? p_rl[i] : size; // Current run-length
@@ -502,13 +421,13 @@ unsigned int o_rng = o_size - 1;
 
       // If any run-lengths are negative (or NA) we stop
       if (rl < 0){
-        Rf_unprotect(NP);
+        YIELD(NP);
         Rf_error("All run lengths must be non-NA and >= 0");
       }
 
       // If the cumulative run-length exceeds length(x) we stop
       if (run_end > size){
-        Rf_unprotect(NP);
+        YIELD(NP);
         Rf_error("sum(run_lengths) must be equal to length(x) (%d)", size);
       }
       // Loop starting from the end of the previous run-length
@@ -516,11 +435,13 @@ unsigned int o_rng = o_size - 1;
         // Check that order value is valid
         if (has_order){
           oi = p_o[j] - 1;
-          check_order_value(oi, o_rng, NP);
+          if (oi < 0 || oi >= size){
+            Rf_error("`order` must be an integer vector of unique values between 1 and `length(x)`");
+          }
         } else {
           oi = j;
         }
-        k = recycle_lag ? p_lag[oi % lag_size] : lag1;
+        k = p_lag[recycle_lag ? oi % lag_size : 0];
         if (k >= 0){
           p_out[oi] = (j - run_start) >= k ? p_x[has_order ? p_o[j - k] - 1 : j - k] : fill_value;
         } else {
@@ -529,10 +450,9 @@ unsigned int o_rng = o_size - 1;
       }
     }
     if (run_end != size){
-      Rf_unprotect(NP);
+      YIELD(NP);
       Rf_error("sum(run_lengths) must be equal to length(x) (%d)", size);
     }
-    CHEAPR_LAG_R_NAMES;
     break;
   }
   case REALSXP: {
@@ -540,13 +460,13 @@ unsigned int o_rng = o_size - 1;
     if (has_order && (size != o_size)){
       Rf_error("length(order) must equal length(x) (%d)", size);
     }
-    double *p_x = REAL(x);
+    const double *p_x = REAL(x);
     double fill_value = NA_REAL;
     if (fill_size >= 1){
       fill_value = Rf_asReal(fill);
     }
-    out = Rf_protect(Rf_duplicate(x)); ++NP;
-    double *p_out = REAL(out);
+    out = SHIELD(cpp_semi_copy(x)); ++NP;
+    double* RESTRICT p_out = REAL(out);
     for (int i = 0; i != rl_size; ++i){
       run_start = run_end; // Start at the end of the previous run
       rl = has_rl ? p_rl[i] : size; // Current run-length
@@ -554,13 +474,13 @@ unsigned int o_rng = o_size - 1;
 
       // If any run-lengths are negative (or NA) we stop
       if (rl < 0){
-        Rf_unprotect(NP);
+        YIELD(NP);
         Rf_error("All run lengths must be non-NA and >= 0");
       }
 
       // If the cumulative run-length exceeds length(x) we stop
       if (run_end > size){
-        Rf_unprotect(NP);
+        YIELD(NP);
         Rf_error("sum(run_lengths) must be equal to length(x) (%d)", size);
       }
       // Loop starting from the end of the previous run-length
@@ -568,11 +488,13 @@ unsigned int o_rng = o_size - 1;
         // Check that order value is valid
         if (has_order){
           oi = p_o[j] - 1;
-          check_order_value(oi, o_rng, NP);
+          if (oi < 0 || oi >= size){
+            Rf_error("`order` must be an integer vector of unique values between 1 and `length(x)`");
+          }
         } else {
           oi = j;
         }
-        k = recycle_lag ? p_lag[oi % lag_size] : lag1;
+        k = p_lag[recycle_lag ? oi % lag_size : 0];
         if (k >= 0){
           p_out[oi] = (j - run_start) >= k ? p_x[has_order ? p_o[j - k] - 1 : j - k] : fill_value;
         } else {
@@ -581,10 +503,9 @@ unsigned int o_rng = o_size - 1;
       }
     }
     if (run_end != size){
-      Rf_unprotect(NP);
+      YIELD(NP);
       Rf_error("sum(run_lengths) must be equal to length(x) (%d)", size);
     }
-    CHEAPR_LAG_R_NAMES;
     break;
   }
   case STRSXP: {
@@ -593,10 +514,8 @@ unsigned int o_rng = o_size - 1;
       Rf_error("length(order) must equal length(x) (%d)", size);
     }
     const SEXP *p_x = STRING_PTR_RO(x);
-    SEXP fill_value = Rf_protect(fill_size >= 1 ? Rf_asChar(fill) : NA_STRING);
-    ++NP;
-    out = Rf_protect(Rf_duplicate(x));
-    ++NP;
+    SEXP fill_value = SHIELD(fill_size >= 1 ? Rf_asChar(fill) : NA_STRING); ++NP;
+    out = SHIELD(cpp_semi_copy(x)); ++NP;
     for (int i = 0; i != rl_size; ++i){
       run_start = run_end; // Start at the end of the previous run
       rl = has_rl ? p_rl[i] : size; // Current run-length
@@ -604,13 +523,13 @@ unsigned int o_rng = o_size - 1;
 
       // If any run-lengths are negative (or NA) we stop
       if (rl < 0){
-        Rf_unprotect(NP);
+        YIELD(NP);
         Rf_error("All run lengths must be non-NA and >= 0");
       }
 
       // If the cumulative run-length exceeds length(x) we stop
       if (run_end > size){
-        Rf_unprotect(NP);
+        YIELD(NP);
         Rf_error("sum(run_lengths) must be equal to length(x) (%d)", size);
       }
       // Loop starting from the end of the previous run-length
@@ -618,11 +537,13 @@ unsigned int o_rng = o_size - 1;
         // Check that order value is valid
         if (has_order){
           oi = p_o[j] - 1;
-          check_order_value(oi, o_rng, NP);
+          if (oi < 0 || oi >= size){
+            Rf_error("`order` must be an integer vector of unique values between 1 and `length(x)`");
+          }
         } else {
           oi = j;
         }
-        k = recycle_lag ? p_lag[oi % lag_size] : lag1;
+        k = p_lag[recycle_lag ? oi % lag_size : 0];
         if (k >= 0){
           SET_STRING_ELT(out, oi, (j - run_start) >= k ? p_x[has_order ? p_o[j - k] - 1 : j - k] : fill_value);
         } else {
@@ -631,10 +552,9 @@ unsigned int o_rng = o_size - 1;
       }
     }
     if (run_end != size){
-      Rf_unprotect(NP);
+      YIELD(NP);
       Rf_error("sum(run_lengths) must be equal to length(x) (%d)", size);
     }
-    CHEAPR_LAG_R_NAMES;
     break;
   }
   case CPLXSXP: {
@@ -643,14 +563,12 @@ unsigned int o_rng = o_size - 1;
       Rf_error("length(order) must equal length(x) (%d)", size);
     }
     Rcomplex *p_x = COMPLEX(x);
-    SEXP fill_sexp = Rf_protect(Rf_allocVector(CPLXSXP, 1));
-    ++NP;
+    SEXP fill_sexp = SHIELD(new_vec(CPLXSXP, 1)); ++NP;
     Rcomplex *p_fill = COMPLEX(fill_sexp);
     p_fill[0].i = NA_REAL;
     p_fill[0].r = NA_REAL;
     Rcomplex fill_value = fill_size >= 1 ? Rf_asComplex(fill) : COMPLEX(fill_sexp)[0];
-    out = Rf_protect(Rf_duplicate(x));
-    ++NP;
+    out = SHIELD(cpp_semi_copy(x)); ++NP;
     for (int i = 0; i != rl_size; ++i){
       run_start = run_end; // Start at the end of the previous run
       rl = has_rl ? p_rl[i] : size; // Current run-length
@@ -658,13 +576,13 @@ unsigned int o_rng = o_size - 1;
 
       // If any run-lengths are negative (or NA) we stop
       if (rl < 0){
-        Rf_unprotect(NP);
+        YIELD(NP);
         Rf_error("All run lengths must be non-NA and >= 0");
       }
 
       // If the cumulative run-length exceeds length(x) we stop
       if (run_end > size){
-        Rf_unprotect(NP);
+        YIELD(NP);
         Rf_error("sum(run_lengths) must be equal to length(x) (%d)", size);
       }
       // Loop starting from the end of the previous run-length
@@ -672,11 +590,13 @@ unsigned int o_rng = o_size - 1;
         // Check that order value is valid
         if (has_order){
           oi = p_o[j] - 1;
-          check_order_value(oi, o_rng, NP);
+          if (oi < 0 || oi >= size){
+            Rf_error("`order` must be an integer vector of unique values between 1 and `length(x)`");
+          }
         } else {
           oi = j;
         }
-        k = recycle_lag ? p_lag[oi % lag_size] : lag1;
+        k = p_lag[recycle_lag ? oi % lag_size : 0];
         if (k >= 0){
           SET_COMPLEX_ELT(out, oi, (j - run_start) >= k ? p_x[has_order ? p_o[j - k] - 1 : j - k] : fill_value);
         } else {
@@ -685,10 +605,9 @@ unsigned int o_rng = o_size - 1;
       }
     }
     if (run_end != size){
-      Rf_unprotect(NP);
+      YIELD(NP);
       Rf_error("sum(run_lengths) must be equal to length(x) (%d)", size);
     }
-    CHEAPR_LAG_R_NAMES;
     break;
   }
   case RAWSXP: {
@@ -697,11 +616,9 @@ unsigned int o_rng = o_size - 1;
       Rf_error("length(order) must equal length(x) (%d)", size);
     }
     Rbyte *p_x = RAW(x);
-    SEXP raw_sexp = Rf_protect(Rf_coerceVector(fill, RAWSXP));
-    Rbyte fill_value = fill_size == 0 ? RAW(Rf_ScalarRaw(0))[0] : RAW(raw_sexp)[0];
-    ++NP;
-    out = Rf_protect(Rf_duplicate(x));
-    ++NP;
+    SEXP raw_sexp = SHIELD(coerce_vec(fill, RAWSXP));
+    Rbyte fill_value = fill_size == 0 ? RAW(Rf_ScalarRaw(0))[0] : RAW(raw_sexp)[0]; ++NP;
+    out = SHIELD(cpp_semi_copy(x)); ++NP;
     for (int i = 0; i != rl_size; ++i){
       run_start = run_end; // Start at the end of the previous run
       rl = has_rl ? p_rl[i] : size; // Current run-length
@@ -709,13 +626,13 @@ unsigned int o_rng = o_size - 1;
 
       // If any run-lengths are negative (or NA) we stop
       if (rl < 0){
-        Rf_unprotect(NP);
+        YIELD(NP);
         Rf_error("All run lengths must be non-NA and >= 0");
       }
 
       // If the cumulative run-length exceeds length(x) we stop
       if (run_end > size){
-        Rf_unprotect(NP);
+        YIELD(NP);
         Rf_error("sum(run_lengths) must be equal to length(x) (%d)", size);
       }
       // Loop starting from the end of the previous run-length
@@ -723,11 +640,13 @@ unsigned int o_rng = o_size - 1;
         // Check that order value is valid
         if (has_order){
           oi = p_o[j] - 1;
-          check_order_value(oi, o_rng, NP);
+          if (oi < 0 || oi >= size){
+            Rf_error("`order` must be an integer vector of unique values between 1 and `length(x)`");
+          }
         } else {
           oi = j;
         }
-        k = recycle_lag ? p_lag[oi % lag_size] : lag1;
+        k = p_lag[recycle_lag ? oi % lag_size : 0];
         if (k >= 0){
           SET_RAW_ELT(out, oi, (j - run_start) >= k ? p_x[has_order ? p_o[j - k] - 1 : j - k] : fill_value);
         } else {
@@ -736,32 +655,19 @@ unsigned int o_rng = o_size - 1;
       }
     }
     if (run_end != size){
-      Rf_unprotect(NP);
+      YIELD(NP);
       Rf_error("sum(run_lengths) must be equal to length(x) (%d)", size);
     }
-    CHEAPR_LAG_R_NAMES;
     break;
   }
   case VECSXP: {
-    if (recursive){
-    int size = Rf_length(x);
-    const SEXP *p_x = VECTOR_PTR_RO(x);
-    out = Rf_protect(Rf_allocVector(VECSXP, size));
-    ++NP;
-    SHALLOW_DUPLICATE_ATTRIB(out, x);
-    for (int i = 0; i < size; ++i){
-      SET_VECTOR_ELT(out, i, cpp_lag2(p_x[i], lag, order, run_lengths, fill, true));
-    }
-  } else {
     int size = Rf_length(x);
     if (has_order && (size != o_size)){
       Rf_error("length(order) must equal length(x) (%d)", size);
     }
     const SEXP *p_x = VECTOR_PTR_RO(x);
-    SEXP fill_value = Rf_protect(VECTOR_ELT(Rf_coerceVector(fill_size >= 1 ? fill : R_NilValue, VECSXP), 0));
-    ++NP;
-    out = Rf_protect(Rf_allocVector(VECSXP, size));
-    ++NP;
+    SEXP fill_value = SHIELD(VECTOR_ELT(coerce_vec(fill_size >= 1 ? fill : R_NilValue, VECSXP), 0)); ++NP;
+    out = SHIELD(new_vec(VECSXP, size)); ++NP;
     for (int i = 0; i != rl_size; ++i){
       run_start = run_end; // Start at the end of the previous run
       rl = has_rl ? p_rl[i] : size; // Current run-length
@@ -769,13 +675,13 @@ unsigned int o_rng = o_size - 1;
 
       // If any run-lengths are negative (or NA) we stop
       if (rl < 0){
-        Rf_unprotect(NP);
+        YIELD(NP);
         Rf_error("All run lengths must be non-NA and >= 0");
       }
 
       // If the cumulative run-length exceeds length(x) we stop
       if (run_end > size){
-        Rf_unprotect(NP);
+        YIELD(NP);
         Rf_error("sum(run_lengths) must be equal to length(x) (%d)", size);
       }
       // Loop starting from the end of the previous run-length
@@ -783,11 +689,13 @@ unsigned int o_rng = o_size - 1;
         // Check that order value is valid
         if (has_order){
           oi = p_o[j] - 1;
-          check_order_value(oi, o_rng, NP);
+          if (oi < 0 || oi >= size){
+            Rf_error("`order` must be an integer vector of unique values between 1 and `length(x)`");
+          }
         } else {
           oi = j;
         }
-        k = recycle_lag ? p_lag[oi % lag_size] : lag1;
+        k = p_lag[recycle_lag ? oi % lag_size : 0];
         if (k >= 0){
           SET_VECTOR_ELT(out, oi, (j - run_start) >= k ? p_x[has_order ? p_o[j - k] - 1 : j - k] : fill_value);
         } else {
@@ -796,18 +704,36 @@ unsigned int o_rng = o_size - 1;
       }
     }
     if (run_end != size){
-      Rf_unprotect(NP);
+      YIELD(NP);
       Rf_error("sum(run_lengths) must be equal to length(x) (%d)", size);
     }
-    CHEAPR_LAG_R_NAMES;
-  }
-  break;
+    break;
   }
   default: {
-    Rf_unprotect(NP);
+    YIELD(NP);
     Rf_error("%s cannot handle an object of type %s", __func__, Rf_type2char(TYPEOF(x)));
   }
   }
-  Rf_unprotect(NP);
+  YIELD(NP);
+  return out;
+}
+
+
+[[cpp11::register]]
+SEXP cpp_lag2(SEXP x, SEXP lag, SEXP order, SEXP run_lengths, SEXP fill, bool recursive){
+  SEXP out = R_NilValue;
+  if (recursive && TYPEOF(x) == VECSXP){
+    R_xlen_t size = Rf_xlength(x);
+    const SEXP *p_x = VECTOR_PTR_RO(x);
+    out = SHIELD(new_vec(VECSXP, size));
+    SHALLOW_DUPLICATE_ATTRIB(out, x);
+    for (R_xlen_t i = 0; i < size; ++i){
+      SET_VECTOR_ELT(out, i, cpp_lag2(p_x[i], lag, order, run_lengths, fill, true));
+    }
+  } else {
+    out = SHIELD(lag2(x, lag, order, run_lengths, fill));
+    set_names(out, lag2(get_names(x), lag, order, run_lengths, fill));
+  }
+  YIELD(1);
   return out;
 }
