@@ -3,7 +3,7 @@
 // Basic math operations by reference
 // All NA and NaN values are ignored
 
-void cpp_check_numeric(SEXP x){
+void check_numeric(SEXP x){
   if (!(Rf_isNumeric(x) && !Rf_isObject(x))){
     Rf_error("x must be a numeric vector");
   }
@@ -45,19 +45,19 @@ for (R_xlen_t i = 0; i < n; ++i) {                                        \
 SEXP convert_int_to_real(SEXP x){
   int *p_x = INTEGER(x);
   R_xlen_t n = Rf_xlength(x);
-  SEXP out = Rf_protect(Rf_allocVector(REALSXP, n));
+  SEXP out = SHIELD(new_vec(REALSXP, n));
   double *p_out = REAL(out);
   for (int i = 0; i < n; ++i){
-    p_out[i] = p_x[i] != NA_INTEGER ? (double)(p_x[i]) : NA_REAL;
+    p_out[i] = p_x[i] != NA_INTEGER ? static_cast<double>(p_x[i]) : NA_REAL;
   }
-  Rf_unprotect(1);
+  YIELD(1);
   return out;
 }
 
 [[cpp11::register]]
 SEXP cpp_set_abs(SEXP x){
-  cpp_check_numeric(x);
-  SEXP out = Rf_protect(check_transform_altrep(x));
+  check_numeric(x);
+  SEXP out = SHIELD(check_transform_altrep(x));
   R_xlen_t n = Rf_xlength(out);
   int n_cores = n >= CHEAPR_OMP_THRESHOLD ? num_cores() : 1;
   switch (TYPEOF(out)){
@@ -85,14 +85,14 @@ SEXP cpp_set_abs(SEXP x){
     break;
   }
   }
-  Rf_unprotect(1);
+  YIELD(1);
   return out;
 }
 
 [[cpp11::register]]
 SEXP cpp_set_floor(SEXP x){
-  cpp_check_numeric(x);
-  SEXP out = Rf_protect(check_transform_altrep(x));
+  check_numeric(x);
+  SEXP out = SHIELD(check_transform_altrep(x));
   R_xlen_t n = Rf_xlength(out);
   int n_cores = n >= CHEAPR_OMP_THRESHOLD ? num_cores() : 1;
   if (Rf_isReal(out)){
@@ -105,14 +105,14 @@ SEXP cpp_set_floor(SEXP x){
       CHEAPR_MATH_REAL_LOOP(std::floor);
     }
   }
-  Rf_unprotect(1);
+  YIELD(1);
   return out;
 }
 
 [[cpp11::register]]
 SEXP cpp_set_ceiling(SEXP x){
-  cpp_check_numeric(x);
-  SEXP out = Rf_protect(check_transform_altrep(x));
+  check_numeric(x);
+  SEXP out = SHIELD(check_transform_altrep(x));
   R_xlen_t n = Rf_xlength(x);
   int n_cores = n >= CHEAPR_OMP_THRESHOLD ? num_cores() : 1;
   if (Rf_isReal(out)){
@@ -125,14 +125,14 @@ SEXP cpp_set_ceiling(SEXP x){
       CHEAPR_MATH_REAL_LOOP(std::ceil);
     }
   }
-  Rf_unprotect(1);
+  YIELD(1);
   return out;
 }
 
 [[cpp11::register]]
 SEXP cpp_set_trunc(SEXP x){
-  cpp_check_numeric(x);
-  SEXP out = Rf_protect(check_transform_altrep(x));
+  check_numeric(x);
+  SEXP out = SHIELD(check_transform_altrep(x));
   R_xlen_t n = Rf_xlength(out);
   int n_cores = n >= CHEAPR_OMP_THRESHOLD ? num_cores() : 1;
   if (Rf_isReal(out)){
@@ -145,14 +145,14 @@ SEXP cpp_set_trunc(SEXP x){
       CHEAPR_MATH_REAL_LOOP(CHEAPR_TRUNC);
     }
   }
-  Rf_unprotect(1);
+  YIELD(1);
   return out;
 }
 
 [[cpp11::register]]
 SEXP cpp_set_change_sign(SEXP x){
-  cpp_check_numeric(x);
-  SEXP out = Rf_protect(check_transform_altrep(x));
+  check_numeric(x);
+  SEXP out = SHIELD(check_transform_altrep(x));
   R_xlen_t n = Rf_xlength(out);
   int n_cores = n >= CHEAPR_OMP_THRESHOLD ? num_cores() : 1;
   switch (TYPEOF(out)){
@@ -188,21 +188,21 @@ SEXP cpp_set_change_sign(SEXP x){
     break;
   }
   }
-  Rf_unprotect(1);
+  YIELD(1);
   return out;
 }
 
 [[cpp11::register]]
 SEXP cpp_set_exp(SEXP x){
-  cpp_check_numeric(x);
+  check_numeric(x);
   R_xlen_t n = Rf_xlength(x);
   int n_cores = n >= CHEAPR_OMP_THRESHOLD ? num_cores() : 1;
   SEXP out;
   if (!Rf_isReal(x)){
     copy_warning();
-    out = Rf_protect(convert_int_to_real(x));
+    out = SHIELD(convert_int_to_real(x));
   } else {
-    out = Rf_protect(check_transform_altrep(x));
+    out = SHIELD(check_transform_altrep(x));
   }
   double *p_out = REAL(out);
   if (n_cores > 1){
@@ -212,21 +212,21 @@ SEXP cpp_set_exp(SEXP x){
     OMP_FOR_SIMD
     CHEAPR_MATH_REAL_LOOP(std::exp);
   }
-  Rf_unprotect(1);
+  YIELD(1);
   return out;
 }
 
 [[cpp11::register]]
 SEXP cpp_set_sqrt(SEXP x){
-  cpp_check_numeric(x);
+  check_numeric(x);
   R_xlen_t n = Rf_xlength(x);
   int n_cores = n >= CHEAPR_OMP_THRESHOLD ? num_cores() : 1;
   SEXP out;
   if (!Rf_isReal(x)){
     copy_warning();
-    out = Rf_protect(convert_int_to_real(x));
+    out = SHIELD(convert_int_to_real(x));
   } else {
-    out = Rf_protect(check_transform_altrep(x));
+    out = SHIELD(check_transform_altrep(x));
   }
   double *p_out = REAL(out);
   if (n_cores > 1){
@@ -236,27 +236,27 @@ SEXP cpp_set_sqrt(SEXP x){
     OMP_FOR_SIMD
     CHEAPR_MATH_REAL_LOOP(std::sqrt);
   }
-  Rf_unprotect(1);
+  YIELD(1);
   return out;
 }
 
 [[cpp11::register]]
 SEXP cpp_set_add(SEXP x, SEXP y){
-  cpp_check_numeric(x);
-  cpp_check_numeric(y);
+  check_numeric(x);
+  check_numeric(y);
   int NP = 0;
-  SEXP out = Rf_protect(check_transform_altrep(x)); ++NP;
+  SEXP out = SHIELD(check_transform_altrep(x)); ++NP;
   R_xlen_t xn = Rf_xlength(out);
   R_xlen_t yn = Rf_xlength(y);
   int n_cores = xn >= CHEAPR_OMP_THRESHOLD ? num_cores() : 1;
 
   if (xn > 0){
     if (yn > xn){
-      Rf_unprotect(NP);
+      YIELD(NP);
       Rf_error("length(y) must be <= length(x)");
     }
     if (yn == 0){
-      Rf_unprotect(NP);
+      YIELD(NP);
       Rf_error("length(y) must be be non-zero");
     }
   }
@@ -279,7 +279,7 @@ SEXP cpp_set_add(SEXP x, SEXP y){
   }
   case REALSXP: {
     copy_warning();
-    Rf_protect(out = Rf_coerceVector(out, REALSXP));
+    SHIELD(out = coerce_vec(out, REALSXP));
     ++NP;
     double *p_x = REAL(out);
     double *p_y = REAL(y);
@@ -322,27 +322,27 @@ SEXP cpp_set_add(SEXP x, SEXP y){
   }
   }
   }
-  Rf_unprotect(NP);
+  YIELD(NP);
   return out;
 }
 
 [[cpp11::register]]
 SEXP cpp_set_subtract(SEXP x, SEXP y){
-  cpp_check_numeric(x);
-  cpp_check_numeric(y);
+  check_numeric(x);
+  check_numeric(y);
   int NP = 0;
-  SEXP out = Rf_protect(check_transform_altrep(x));
+  SEXP out = SHIELD(check_transform_altrep(x));
   ++NP;
   R_xlen_t xn = Rf_xlength(out);
   R_xlen_t yn = Rf_xlength(y);
   int n_cores = xn >= CHEAPR_OMP_THRESHOLD ? num_cores() : 1;
   if (xn > 0){
     if (yn > xn){
-      Rf_unprotect(NP);
+      YIELD(NP);
       Rf_error("length(y) must be <= length(x)");
     }
     if (yn == 0){
-      Rf_unprotect(NP);
+      YIELD(NP);
       Rf_error("length(y) must be be non-zero");
     }
   }
@@ -364,7 +364,7 @@ SEXP cpp_set_subtract(SEXP x, SEXP y){
   }
   case REALSXP: {
     copy_warning();
-    Rf_protect(out = Rf_coerceVector(out, REALSXP));
+    SHIELD(out = coerce_vec(out, REALSXP));
     ++NP;
     double *p_x = REAL(out);
     double *p_y = REAL(y);
@@ -407,27 +407,27 @@ SEXP cpp_set_subtract(SEXP x, SEXP y){
   }
   }
   }
-  Rf_unprotect(NP);
+  YIELD(NP);
   return out;
 }
 
 [[cpp11::register]]
 SEXP cpp_set_multiply(SEXP x, SEXP y){
-  cpp_check_numeric(x);
-  cpp_check_numeric(y);
+  check_numeric(x);
+  check_numeric(y);
   int NP = 0;
-  SEXP out = Rf_protect(check_transform_altrep(x));
+  SEXP out = SHIELD(check_transform_altrep(x));
   ++NP;
   R_xlen_t xn = Rf_xlength(out);
   R_xlen_t yn = Rf_xlength(y);
   int n_cores = xn >= CHEAPR_OMP_THRESHOLD ? num_cores() : 1;
   if (xn > 0){
     if (yn > xn){
-      Rf_unprotect(NP);
+      YIELD(NP);
       Rf_error("length(y) must be <= length(x)");
     }
     if (yn == 0){
-      Rf_unprotect(NP);
+      YIELD(NP);
       Rf_error("length(y) must be be non-zero");
     }
   }
@@ -449,7 +449,7 @@ SEXP cpp_set_multiply(SEXP x, SEXP y){
   }
   case REALSXP: {
     copy_warning();
-    Rf_protect(out = Rf_coerceVector(out, REALSXP));
+    SHIELD(out = coerce_vec(out, REALSXP));
     ++NP;
     double *p_x = REAL(out);
     double *p_y = REAL(y);
@@ -468,7 +468,7 @@ SEXP cpp_set_multiply(SEXP x, SEXP y){
     switch (TYPEOF(y)){
   case LGLSXP:
   case INTSXP: {
-    // Rf_protect(y = Rf_coerceVector(y, REALSXP));
+    // SHIELD(y = coerce_vec(y, REALSXP));
     // ++NP;
     double *p_x = REAL(out);
     int *p_y = INTEGER(y);
@@ -494,14 +494,14 @@ SEXP cpp_set_multiply(SEXP x, SEXP y){
   }
   }
   }
-  Rf_unprotect(NP);
+  YIELD(NP);
   return out;
 }
 
 [[cpp11::register]]
 SEXP cpp_set_divide(SEXP x, SEXP y){
-  cpp_check_numeric(x);
-  cpp_check_numeric(y);
+  check_numeric(x);
+  check_numeric(y);
   R_xlen_t xn = Rf_xlength(x);
   R_xlen_t yn = Rf_xlength(y);
   int n_cores = xn >= CHEAPR_OMP_THRESHOLD ? num_cores() : 1;
@@ -516,9 +516,9 @@ SEXP cpp_set_divide(SEXP x, SEXP y){
   SEXP out;
   if (!Rf_isReal(x)){
     copy_warning();
-    out = Rf_protect(convert_int_to_real(x));
+    out = SHIELD(convert_int_to_real(x));
   } else {
-    out = Rf_protect(check_transform_altrep(x));
+    out = SHIELD(check_transform_altrep(x));
   }
   switch (TYPEOF(y)){
   case LGLSXP:
@@ -545,14 +545,14 @@ SEXP cpp_set_divide(SEXP x, SEXP y){
     break;
   }
   }
-  Rf_unprotect(1);
+  YIELD(1);
   return out;
 }
 
 [[cpp11::register]]
 SEXP cpp_set_pow(SEXP x, SEXP y){
-  cpp_check_numeric(x);
-  cpp_check_numeric(y);
+  check_numeric(x);
+  check_numeric(y);
   R_xlen_t xn = Rf_xlength(x);
   R_xlen_t yn = Rf_xlength(y);
   int n_cores = xn >= CHEAPR_OMP_THRESHOLD ? num_cores() : 1;
@@ -567,9 +567,9 @@ SEXP cpp_set_pow(SEXP x, SEXP y){
   SEXP out;
   if (!Rf_isReal(x)){
     copy_warning();
-    out = Rf_protect(convert_int_to_real(x));
+    out = SHIELD(convert_int_to_real(x));
   } else {
-    out = Rf_protect(check_transform_altrep(x));
+    out = SHIELD(check_transform_altrep(x));
   }
   switch (TYPEOF(y)){
   case INTSXP: {
@@ -603,14 +603,14 @@ SEXP cpp_set_pow(SEXP x, SEXP y){
     break;
   }
   }
-  Rf_unprotect(1);
+  YIELD(1);
   return out;
 }
 
 [[cpp11::register]]
 SEXP cpp_set_log(SEXP x, SEXP base){
-  cpp_check_numeric(x);
-  cpp_check_numeric(base);
+  check_numeric(x);
+  check_numeric(base);
   R_xlen_t xn = Rf_xlength(x);
   R_xlen_t basen = Rf_xlength(base);
   if (xn > 0){
@@ -625,9 +625,9 @@ SEXP cpp_set_log(SEXP x, SEXP base){
   SEXP out;
   if (!Rf_isReal(x)){
     copy_warning();
-    out = Rf_protect(convert_int_to_real(x));
+    out = SHIELD(convert_int_to_real(x));
   } else {
-    out = Rf_protect(check_transform_altrep(x));
+    out = SHIELD(check_transform_altrep(x));
   }
   double *p_x = REAL(out);
   double *p_base = REAL(base);
@@ -646,15 +646,15 @@ SEXP cpp_set_log(SEXP x, SEXP base){
       NA_REAL : std::log(p_x[i]) / std::log(p_base[basei]);
     }
   }
-  Rf_unprotect(1);
+  YIELD(1);
   return out;
 }
 
 [[cpp11::register]]
 SEXP cpp_set_round(SEXP x, SEXP digits){
-  cpp_check_numeric(x);
-  cpp_check_numeric(digits);
-  SEXP out = Rf_protect(check_transform_altrep(x));
+  check_numeric(x);
+  check_numeric(digits);
+  SEXP out = SHIELD(check_transform_altrep(x));
   R_xlen_t xn = Rf_xlength(out);
   R_xlen_t digitsn = Rf_xlength(digits);
   if (xn > 0){
@@ -747,25 +747,25 @@ SEXP cpp_set_round(SEXP x, SEXP digits){
     }
     }
   }
-  Rf_unprotect(1);
+  YIELD(1);
   return out;
 }
 
 [[cpp11::register]]
 SEXP cpp_int_sign(SEXP x){
-  cpp_check_numeric(x);
+  check_numeric(x);
   R_xlen_t n = Rf_xlength(x);
-  SEXP out = Rf_protect(Rf_allocVector(INTSXP, n));
-  int *p_out = INTEGER(out);
+  SEXP out = SHIELD(new_vec(INTSXP, n));
+  int* RESTRICT p_out = INTEGER(out);
   int res;
   switch (TYPEOF(x)){
   case LGLSXP: {
-    int *p_x = INTEGER(x);
+    const int *p_x = LOGICAL(x);
     memmove(&p_out[0], &p_x[0], n * sizeof(int));
     break;
   }
   case INTSXP: {
-    int *p_x = INTEGER(x);
+    const int *p_x = INTEGER(x);
     OMP_FOR_SIMD
     for (R_xlen_t i = 0; i < n; ++i) {
       res = p_x[i] == NA_INTEGER ? NA_INTEGER : (p_x[i] > 0) - (p_x[i] < 0);
@@ -783,6 +783,6 @@ SEXP cpp_int_sign(SEXP x){
     break;
   }
   }
-  Rf_unprotect(1);
+  YIELD(1);
   return out;
 }
