@@ -5,14 +5,14 @@
 
 // Count the number of true values
 
-R_xlen_t count_true(int *px, R_xlen_t n){
+R_xlen_t count_true(const int* RESTRICT px, R_xlen_t n){
   R_xlen_t size = 0;
   if (n >= CHEAPR_OMP_THRESHOLD){
 #pragma omp parallel for simd num_threads(num_cores()) reduction(+:size)
     for (R_xlen_t j = 0; j < n; ++j) size += (px[j] == TRUE);
     return size;
   } else {
-#pragma omp for simd
+    OMP_FOR_SIMD
     for (R_xlen_t j = 0; j < n; ++j) size += (px[j] == TRUE);
     return size;
   }
@@ -33,48 +33,48 @@ while (whichi < out_size){                                     \
 [[cpp11::register]]
 SEXP cpp_which_(SEXP x, bool invert){
   R_xlen_t n = Rf_xlength(x);
-  int *p_x = LOGICAL(x);
+  const int *p_x = LOGICAL(x);
   bool is_long = (n > integer_max_);
   if (invert){
     if (is_long){
       R_xlen_t size = count_true(p_x, n);
       R_xlen_t out_size = n - size;
-      SEXP out = Rf_protect(Rf_allocVector(REALSXP, out_size));
-      double *p_out = REAL(out);
+      SEXP out = SHIELD(new_vec(REALSXP, out_size));
+      double* RESTRICT p_out = REAL(out);
       R_xlen_t whichi = 0;
       R_xlen_t i = 0;
       CHEAPR_WHICH_VAL_INVERTED(TRUE);
-      Rf_unprotect(1);
+      YIELD(1);
       return out;
     } else {
       int size = count_true(p_x, n);
       int out_size = n - size;
-      SEXP out = Rf_protect(Rf_allocVector(INTSXP, out_size));
-      int *p_out = INTEGER(out);
+      SEXP out = SHIELD(new_vec(INTSXP, out_size));
+      int* RESTRICT p_out = INTEGER(out);
       int whichi = 0;
       int i = 0;
       CHEAPR_WHICH_VAL_INVERTED(TRUE);
-      Rf_unprotect(1);
+      YIELD(1);
       return out;
     }
   } else {
     if (is_long){
       R_xlen_t out_size = count_true(p_x, n);
-      SEXP out = Rf_protect(Rf_allocVector(REALSXP, out_size));
-      double *p_out = REAL(out);
+      SEXP out = SHIELD(new_vec(REALSXP, out_size));
+      double* RESTRICT p_out = REAL(out);
       R_xlen_t whichi = 0;
       R_xlen_t i = 0;
       CHEAPR_WHICH_VAL(TRUE);
-      Rf_unprotect(1);
+      YIELD(1);
       return out;
     } else {
       int out_size = count_true(p_x, n);
-      SEXP out = Rf_protect(Rf_allocVector(INTSXP, out_size));
-      int *p_out = INTEGER(out);
+      SEXP out = SHIELD(new_vec(INTSXP, out_size));
+      int* RESTRICT p_out = INTEGER(out);
       int whichi = 0;
       int i = 0;
       CHEAPR_WHICH_VAL(TRUE);
-      Rf_unprotect(1);
+      YIELD(1);
       return out;
     }
   }
@@ -88,21 +88,24 @@ SEXP cpp_which_val(SEXP x, SEXP value, bool invert){
   if (Rf_length(value) != 1){
     Rf_error("value must be a vector of length 1");
   }
-  if (Rf_isVectorList(x)){
+  if (TYPEOF(x) == VECSXP){
     Rf_error("%s cannot handle an object of type %s", __func__, Rf_type2char(TYPEOF(x)));
   }
-  SEXP val_is_na = Rf_protect(cpp_is_na(value)); ++NP;
+  SEXP val_is_na = SHIELD(cpp_is_na(value)); ++NP;
   if (Rf_asLogical(val_is_na)){
-    Rf_unprotect(NP);
     if (invert){
-      return cpp_which_not_na(x);
+      SEXP out = SHIELD(cpp_which_not_na(x)); ++NP;
+      YIELD(NP);
+      return out;
     } else {
-      return cpp_which_na(x);
+      SEXP out = SHIELD(cpp_which_na(x)); ++NP;
+      YIELD(NP);
+      return out;
     }
   }
 
   if (implicit_na_coercion(value, x)){
-    Rf_unprotect(NP);
+    YIELD(NP);
     Rf_error("Value has been implicitly converted to NA, please check");
   }
 
@@ -113,82 +116,78 @@ SEXP cpp_which_val(SEXP x, SEXP value, bool invert){
   switch ( CHEAPR_TYPEOF(x) ){
   case LGLSXP:
   case INTSXP: {
-    SEXP out = Rf_protect(Rf_allocVector(is_long ? REALSXP : INTSXP, out_size));
-    ++NP;
-    Rf_protect(value = coerce_vector(value, CHEAPR_TYPEOF(x))); ++NP;
+    SEXP out = SHIELD(new_vec(is_long ? REALSXP : INTSXP, out_size)); ++NP;
+    SHIELD(value = coerce_vector(value, CHEAPR_TYPEOF(x))); ++NP;
     int val = Rf_asInteger(value);
-    int *p_x = INTEGER(x);
+    const int *p_x = INTEGER(x);
     if (is_long){
-      double *p_out = REAL(out);
+      double* RESTRICT p_out = REAL(out);
       if (invert){
         CHEAPR_WHICH_VAL_INVERTED(val);
       } else {
         CHEAPR_WHICH_VAL(val);
       }
     } else {
-      int *p_out = INTEGER(out);
+      int* RESTRICT p_out = INTEGER(out);
       if (invert){
         CHEAPR_WHICH_VAL_INVERTED(val);
       } else {
         CHEAPR_WHICH_VAL(val);
       }
     }
-    Rf_unprotect(NP);
+    YIELD(NP);
     return out;
   }
   case REALSXP: {
-    SEXP out = Rf_protect(Rf_allocVector(is_long ? REALSXP : INTSXP, out_size));
-    ++NP;
-    Rf_protect(value = coerce_vector(value, REALSXP)); ++NP;
+    SEXP out = SHIELD(new_vec(is_long ? REALSXP : INTSXP, out_size)); ++NP;
+    SHIELD(value = coerce_vector(value, REALSXP)); ++NP;
     double val = Rf_asReal(value);
-    double *p_x = REAL(x);
+    const double *p_x = REAL(x);
     if (is_long){
-      double *p_out = REAL(out);
+      double* RESTRICT p_out = REAL(out);
       if (invert){
         CHEAPR_WHICH_VAL_INVERTED(val);
       } else {
         CHEAPR_WHICH_VAL(val);
       }
     } else {
-      int *p_out = INTEGER(out);
+      int* RESTRICT p_out = INTEGER(out);
       if (invert){
         CHEAPR_WHICH_VAL_INVERTED(val);
       } else {
         CHEAPR_WHICH_VAL(val);
       }
     }
-    Rf_unprotect(NP);
+    YIELD(NP);
     return out;
   }
   case CHEAPR_INT64SXP: {
-    SEXP out = Rf_protect(Rf_allocVector(is_long ? REALSXP : INTSXP, out_size));
-    ++NP;
-    Rf_protect(value = coerce_vector(value, CHEAPR_INT64SXP)); ++NP;
-    long long int val = INTEGER64_PTR(value)[0];
-    long long int *p_x = INTEGER64_PTR(x);
+    SEXP out = SHIELD(new_vec(is_long ? REALSXP : INTSXP, out_size)); ++NP;
+    SHIELD(value = coerce_vector(value, CHEAPR_INT64SXP)); ++NP;
+    int_fast64_t val = INTEGER64_PTR(value)[0];
+    const int_fast64_t *p_x = INTEGER64_PTR(x);
     if (is_long){
-      double *p_out = REAL(out);
+      double* RESTRICT p_out = REAL(out);
       if (invert){
         CHEAPR_WHICH_VAL_INVERTED(val);
       } else {
         CHEAPR_WHICH_VAL(val);
       }
     } else {
-      int *p_out = INTEGER(out);
+      int* RESTRICT p_out = INTEGER(out);
       if (invert){
         CHEAPR_WHICH_VAL_INVERTED(val);
       } else {
         CHEAPR_WHICH_VAL(val);
       }
     }
-    Rf_unprotect(NP);
+    YIELD(NP);
     return out;
   }
   case STRSXP: {
-    SEXP out = Rf_protect(Rf_allocVector(is_long ? REALSXP : INTSXP, out_size));
-    ++NP;
-    Rf_protect(value = coerce_vector(value, STRSXP)); ++NP;
-    SEXP val = Rf_protect(Rf_asChar(value)); ++NP;
+    SEXP out = SHIELD(new_vec(is_long ? REALSXP : INTSXP, out_size)); ++NP;
+    SHIELD(value = coerce_vector(value, STRSXP)); ++NP;
+    SEXP val = SHIELD(Rf_asChar(value)); ++NP;
     const SEXP *p_x = STRING_PTR_RO(x);
     if (is_long){
       double *p_out = REAL(out);
@@ -205,11 +204,11 @@ SEXP cpp_which_val(SEXP x, SEXP value, bool invert){
         CHEAPR_WHICH_VAL(val);
       }
     }
-    Rf_unprotect(NP);
+    YIELD(NP);
     return out;
   }
   default: {
-    Rf_unprotect(NP);
+    YIELD(NP);
     Rf_error("%s cannot handle an object of type %s", __func__, Rf_type2char(TYPEOF(x)));
   }
   }
@@ -223,70 +222,68 @@ SEXP cpp_which_na(SEXP x){
   bool is_short = (n <= integer_max_);
   switch ( CHEAPR_TYPEOF(x) ){
   case NILSXP: {
-    SEXP out = Rf_protect(Rf_allocVector(INTSXP, 0));
-    Rf_unprotect(1);
-    return out;
+    return new_vec(INTSXP, 0);
   }
   case LGLSXP:
   case INTSXP: {
     R_xlen_t count = na_count(x, true);
-    int *p_x = INTEGER(x);
+    const int *p_x = INTEGER(x);
     if (is_short){
       int out_size = count;
-      SEXP out = Rf_protect(Rf_allocVector(INTSXP, out_size));
-      int *p_out = INTEGER(out);
+      SEXP out = SHIELD(new_vec(INTSXP, out_size));
+      int* RESTRICT p_out = INTEGER(out);
       int whichi = 0;
       int i = 0;
       CHEAPR_WHICH_VAL(NA_INTEGER);
-      Rf_unprotect(1);
+      YIELD(1);
       return out;
     } else {
       R_xlen_t out_size = count;
-      SEXP out = Rf_protect(Rf_allocVector(REALSXP, out_size));
-      double *p_out = REAL(out);
+      SEXP out = SHIELD(new_vec(REALSXP, out_size));
+      double* RESTRICT p_out = REAL(out);
       R_xlen_t whichi = 0;
       R_xlen_t i = 0;
       CHEAPR_WHICH_VAL(NA_INTEGER);
-      Rf_unprotect(1);
+      YIELD(1);
       return out;
     }
   }
   case CHEAPR_INT64SXP: {
     R_xlen_t count = na_count(x, true);
-    long long *p_x = INTEGER64_PTR(x);
+    const int_fast64_t *p_x = INTEGER64_PTR(x);
     if (is_short){
       int out_size = count;
-      SEXP out = Rf_protect(Rf_allocVector(INTSXP, out_size));
-      int *p_out = INTEGER(out);
+      SEXP out = SHIELD(new_vec(INTSXP, out_size));
+      int* RESTRICT p_out = INTEGER(out);
       int whichi = 0;
       int i = 0;
       while (whichi < out_size){
         p_out[whichi] = i + 1;
         whichi += (p_x[i++] == NA_INTEGER64);
       }
-      Rf_unprotect(1);
+      YIELD(1);
       return out;
     } else {
       R_xlen_t out_size = count;
-      SEXP out = Rf_protect(Rf_allocVector(REALSXP, out_size));
-      double *p_out = REAL(out);
+      SEXP out = SHIELD(new_vec(REALSXP, out_size));
+      double* RESTRICT p_out = REAL(out);
       R_xlen_t whichi = 0;
       R_xlen_t i = 0;
       while (whichi < out_size){
         p_out[whichi] = i + 1;
         whichi += (p_x[i++] == NA_INTEGER64);
       }
-      Rf_unprotect(1);
+      YIELD(1);
       return out;
     }
   }
   case REALSXP: {
     R_xlen_t count = na_count(x, true);
-    double *p_x = REAL(x);
+    const double *p_x = REAL(x);
     if (is_short){
       int out_size = count;
-      SEXP out = Rf_protect(Rf_allocVector(INTSXP, out_size));
-      int *p_out = INTEGER(out);
+      SEXP out = SHIELD(new_vec(INTSXP, out_size));
+      int* RESTRICT p_out = INTEGER(out);
       int whichi = 0;
       int i = 0;
       while (whichi < out_size){
@@ -294,12 +291,12 @@ SEXP cpp_which_na(SEXP x){
         whichi += (p_x[i] != p_x[i]);
         ++i;
       }
-      Rf_unprotect(1);
+      YIELD(1);
       return out;
     } else {
       R_xlen_t out_size = count;
-      SEXP out = Rf_protect(Rf_allocVector(REALSXP, out_size));
-      double *p_out = REAL(out);
+      SEXP out = SHIELD(new_vec(REALSXP, out_size));
+      double* RESTRICT p_out = REAL(out);
       R_xlen_t whichi = 0;
       R_xlen_t i = 0;
       while (whichi < out_size){
@@ -307,7 +304,7 @@ SEXP cpp_which_na(SEXP x){
         whichi += (p_x[i] != p_x[i]);
         ++i;
       }
-      Rf_unprotect(1);
+      YIELD(1);
       return out;
     }
   }
@@ -316,27 +313,27 @@ SEXP cpp_which_na(SEXP x){
     const SEXP *p_x = STRING_PTR_RO(x);
     if (is_short){
       int out_size = count;
-      SEXP out = Rf_protect(Rf_allocVector(INTSXP, out_size));
+      SEXP out = SHIELD(new_vec(INTSXP, out_size));
       int *p_out = INTEGER(out);
       int whichi = 0;
       int i = 0;
       CHEAPR_WHICH_VAL(NA_STRING);
-      Rf_unprotect(1);
+      YIELD(1);
       return out;
     } else {
       R_xlen_t out_size = count;
-      SEXP out = Rf_protect(Rf_allocVector(REALSXP, out_size));
+      SEXP out = SHIELD(new_vec(REALSXP, out_size));
       double *p_out = REAL(out);
       R_xlen_t whichi = 0;
       R_xlen_t i = 0;
       CHEAPR_WHICH_VAL(NA_STRING);
-      Rf_unprotect(1);
+      YIELD(1);
       return out;
     }
   }
   case RAWSXP: {
-    SEXP out = Rf_protect(Rf_allocVector(INTSXP, 0));
-    Rf_unprotect(1);
+    SEXP out = SHIELD(new_vec(INTSXP, 0));
+    YIELD(1);
     return out;
   }
   case CPLXSXP: {
@@ -344,36 +341,36 @@ SEXP cpp_which_na(SEXP x){
     Rcomplex *p_x = COMPLEX(x);
     if (is_short){
       int out_size = count;
-      SEXP out = Rf_protect(Rf_allocVector(INTSXP, out_size));
+      SEXP out = SHIELD(new_vec(INTSXP, out_size));
       int *p_out = INTEGER(out);
       int whichi = 0;
       int i = 0;
       while (whichi < out_size){
         p_out[whichi] = i + 1;
-        whichi += cheapr_is_na_cplx(p_x[i]);
+        whichi += is_na_cplx(p_x[i]);
         ++i;
       }
-      Rf_unprotect(1);
+      YIELD(1);
       return out;
     } else {
       R_xlen_t out_size = count;
-      SEXP out = Rf_protect(Rf_allocVector(REALSXP, out_size));
+      SEXP out = SHIELD(new_vec(REALSXP, out_size));
       double *p_out = REAL(out);
       R_xlen_t whichi = 0;
       R_xlen_t i = 0;
       while (whichi < out_size){
         p_out[whichi] = i + 1;
-        whichi += cheapr_is_na_cplx(p_x[i]);
+        whichi += is_na_cplx(p_x[i]);
         ++i;
       }
-      Rf_unprotect(1);
+      YIELD(1);
       return out;
     }
   }
   default: {
-    SEXP is_missing = Rf_protect(cheapr_is_na(x));
-    SEXP out = Rf_protect(cpp_which_(is_missing, false));
-    Rf_unprotect(2);
+    SEXP is_missing = SHIELD(cheapr_is_na(x));
+    SEXP out = SHIELD(cpp_which_(is_missing, false));
+    YIELD(2);
     return out;
   }
   }
@@ -385,70 +382,68 @@ SEXP cpp_which_not_na(SEXP x){
   bool is_short = (n <= integer_max_);
   switch ( CHEAPR_TYPEOF(x) ){
   case NILSXP: {
-    SEXP out = Rf_protect(Rf_allocVector(INTSXP, 0));
-    Rf_unprotect(1);
-    return out;
+    return new_vec(INTSXP, 0);
   }
   case LGLSXP:
   case INTSXP: {
     R_xlen_t count = na_count(x, true);
-    int *p_x = INTEGER(x);
+    const int *p_x = INTEGER(x);
     if (is_short){
       int out_size = n - count;
-      SEXP out = Rf_protect(Rf_allocVector(INTSXP, out_size));
-      int *p_out = INTEGER(out);
+      SEXP out = SHIELD(new_vec(INTSXP, out_size));
+      int* RESTRICT p_out = INTEGER(out);
       int whichi = 0;
       int i = 0;
       CHEAPR_WHICH_VAL_INVERTED(NA_INTEGER);
-      Rf_unprotect(1);
+      YIELD(1);
       return out;
     } else {
       R_xlen_t out_size = n - count;
-      SEXP out = Rf_protect(Rf_allocVector(REALSXP, out_size));
-      double *p_out = REAL(out);
+      SEXP out = SHIELD(new_vec(REALSXP, out_size));
+      double* RESTRICT p_out = REAL(out);
       R_xlen_t whichi = 0;
       R_xlen_t i = 0;
       CHEAPR_WHICH_VAL_INVERTED(NA_INTEGER);
-      Rf_unprotect(1);
+      YIELD(1);
       return out;
     }
   }
   case CHEAPR_INT64SXP: {
     R_xlen_t count = na_count(x, true);
-    long long *p_x = INTEGER64_PTR(x);
+    const int_fast64_t *p_x = INTEGER64_PTR(x);
     if (is_short){
       int out_size = n - count;
-      SEXP out = Rf_protect(Rf_allocVector(INTSXP, out_size));
-      int *p_out = INTEGER(out);
+      SEXP out = SHIELD(new_vec(INTSXP, out_size));
+      int* RESTRICT p_out = INTEGER(out);
       int whichi = 0;
       int i = 0;
       while (whichi < out_size){
         p_out[whichi] = i + 1;
         whichi += (p_x[i++] != NA_INTEGER64);
       }
-      Rf_unprotect(1);
+      YIELD(1);
       return out;
     } else {
       R_xlen_t out_size = n - count;
-      SEXP out = Rf_protect(Rf_allocVector(REALSXP, out_size));
-      double *p_out = REAL(out);
+      SEXP out = SHIELD(new_vec(REALSXP, out_size));
+      double* RESTRICT p_out = REAL(out);
       R_xlen_t whichi = 0;
       R_xlen_t i = 0;
       while (whichi < out_size){
         p_out[whichi] = i + 1;
         whichi += (p_x[i++] != NA_INTEGER64);
       }
-      Rf_unprotect(1);
+      YIELD(1);
       return out;
     }
   }
   case REALSXP: {
     R_xlen_t count = na_count(x, true);
-    double *p_x = REAL(x);
+    const double *p_x = REAL(x);
     if (is_short){
       int out_size = n - count;
-      SEXP out = Rf_protect(Rf_allocVector(INTSXP, out_size));
-      int *p_out = INTEGER(out);
+      SEXP out = SHIELD(new_vec(INTSXP, out_size));
+      int* RESTRICT p_out = INTEGER(out);
       int whichi = 0;
       int i = 0;
       while (whichi < out_size){
@@ -456,12 +451,12 @@ SEXP cpp_which_not_na(SEXP x){
         whichi += (p_x[i] == p_x[i]);
         ++i;
       }
-      Rf_unprotect(1);
+      YIELD(1);
       return out;
     } else {
       R_xlen_t out_size = n - count;
-      SEXP out = Rf_protect(Rf_allocVector(REALSXP, out_size));
-      double *p_out = REAL(out);
+      SEXP out = SHIELD(new_vec(REALSXP, out_size));
+      double* RESTRICT p_out = REAL(out);
       R_xlen_t whichi = 0;
       R_xlen_t i = 0;
       while (whichi < out_size){
@@ -469,7 +464,7 @@ SEXP cpp_which_not_na(SEXP x){
         whichi += (p_x[i] == p_x[i]);
         ++i;
       }
-      Rf_unprotect(1);
+      YIELD(1);
       return out;
     }
   }
@@ -478,40 +473,40 @@ SEXP cpp_which_not_na(SEXP x){
     const SEXP *p_x = STRING_PTR_RO(x);
     if (is_short){
       int out_size = n - count;
-      SEXP out = Rf_protect(Rf_allocVector(INTSXP, out_size));
-      int *p_out = INTEGER(out);
+      SEXP out = SHIELD(new_vec(INTSXP, out_size));
+      int* RESTRICT p_out = INTEGER(out);
       int whichi = 0;
       int i = 0;
       CHEAPR_WHICH_VAL_INVERTED(NA_STRING);
-      Rf_unprotect(1);
+      YIELD(1);
       return out;
     } else {
       R_xlen_t out_size = n - count;
-      SEXP out = Rf_protect(Rf_allocVector(REALSXP, out_size));
-      double *p_out = REAL(out);
+      SEXP out = SHIELD(new_vec(REALSXP, out_size));
+      double* RESTRICT p_out = REAL(out);
       R_xlen_t whichi = 0;
       R_xlen_t i = 0;
       CHEAPR_WHICH_VAL_INVERTED(NA_STRING);
-      Rf_unprotect(1);
+      YIELD(1);
       return out;
     }
   }
   case RAWSXP: {
     if (is_short){
-    SEXP out = Rf_protect(Rf_allocVector(INTSXP, n));
-    int *p_out = INTEGER(out);
+    SEXP out = SHIELD(new_vec(INTSXP, n));
+    int* RESTRICT p_out = INTEGER(out);
     for (int i = 0; i < n; ++i){
       p_out[i] = i + 1;
     }
-    Rf_unprotect(1);
+    YIELD(1);
     return out;
   } else {
-    SEXP out = Rf_protect(Rf_allocVector(REALSXP, n));
-    double *p_out = REAL(out);
+    SEXP out = SHIELD(new_vec(REALSXP, n));
+    double* RESTRICT p_out = REAL(out);
     for (R_xlen_t i = 0; i < n; ++i){
       p_out[i] = i + 1;
     }
-    Rf_unprotect(1);
+    YIELD(1);
     return out;
   }
   }
@@ -520,36 +515,36 @@ SEXP cpp_which_not_na(SEXP x){
     Rcomplex *p_x = COMPLEX(x);
     if (is_short){
       int out_size = n - count;
-      SEXP out = Rf_protect(Rf_allocVector(INTSXP, out_size));
-      int *p_out = INTEGER(out);
+      SEXP out = SHIELD(new_vec(INTSXP, out_size));
+      int* RESTRICT p_out = INTEGER(out);
       int whichi = 0;
       int i = 0;
       while (whichi < out_size){
         p_out[whichi] = i + 1;
-        whichi += !cheapr_is_na_cplx(p_x[i]);
+        whichi += !is_na_cplx(p_x[i]);
         ++i;
       }
-      Rf_unprotect(1);
+      YIELD(1);
       return out;
     } else {
       R_xlen_t out_size = n - count;
-      SEXP out = Rf_protect(Rf_allocVector(REALSXP, out_size));
-      double *p_out = REAL(out);
+      SEXP out = SHIELD(new_vec(REALSXP, out_size));
+      double* RESTRICT p_out = REAL(out);
       R_xlen_t whichi = 0;
       R_xlen_t i = 0;
       while (whichi < out_size){
         p_out[whichi] = i + 1;
-        whichi += !cheapr_is_na_cplx(p_x[i]);
+        whichi += !is_na_cplx(p_x[i]);
         ++i;
       }
-      Rf_unprotect(1);
+      YIELD(1);
       return out;
     }
   }
   default: {
-    SEXP is_missing = Rf_protect(cheapr_is_na(x));
-    SEXP out = Rf_protect(cpp_which_(is_missing, true));
-    Rf_unprotect(2);
+    SEXP is_missing = SHIELD(cheapr_is_na(x));
+    SEXP out = SHIELD(cpp_which_(is_missing, true));
+    YIELD(2);
     return out;
   }
   }
@@ -565,9 +560,9 @@ SEXP cpp_lgl_locs(SEXP x, R_xlen_t n_true, R_xlen_t n_false,
   int *p_x = LOGICAL(x);
 
   if (n > integer_max_){
-    SEXP true_locs = Rf_protect(Rf_allocVector(REALSXP, include_true ? n_true : 0));
-    SEXP false_locs = Rf_protect(Rf_allocVector(REALSXP, include_false ? n_false : 0));
-    SEXP na_locs = Rf_protect(Rf_allocVector(REALSXP, include_na ? (n - n_true - n_false) : 0));
+    SEXP true_locs = SHIELD(new_vec(REALSXP, include_true ? n_true : 0));
+    SEXP false_locs = SHIELD(new_vec(REALSXP, include_false ? n_false : 0));
+    SEXP na_locs = SHIELD(new_vec(REALSXP, include_na ? (n - n_true - n_false) : 0));
 
     double *p_true = REAL(true_locs);
     double *p_false = REAL(false_locs);
@@ -586,23 +581,23 @@ SEXP cpp_lgl_locs(SEXP x, R_xlen_t n_true, R_xlen_t n_false,
         p_na[k3++] = i + 1;
       }
     }
-    SEXP out = Rf_protect(Rf_allocVector(VECSXP, 3));
+    SEXP out = SHIELD(new_vec(VECSXP, 3));
     SET_VECTOR_ELT(out, 0, true_locs);
     SET_VECTOR_ELT(out, 1, false_locs);
     SET_VECTOR_ELT(out, 2, na_locs);
 
-    SEXP names = Rf_protect(Rf_allocVector(STRSXP, 3));
-    SET_STRING_ELT(names, 0, Rf_mkChar("true"));
-    SET_STRING_ELT(names, 1, Rf_mkChar("false"));
-    SET_STRING_ELT(names, 2, Rf_mkChar("na"));
-    Rf_setAttrib(out, R_NamesSymbol, names);
+    SEXP names = SHIELD(new_vec(STRSXP, 3));
+    SET_STRING_ELT(names, 0, make_utf8_char("true"));
+    SET_STRING_ELT(names, 1, make_utf8_char("false"));
+    SET_STRING_ELT(names, 2, make_utf8_char("na"));
+    set_names(out, names);
 
-    Rf_unprotect(5);
+    YIELD(5);
     return out;
   } else {
-    SEXP true_locs = Rf_protect(Rf_allocVector(INTSXP, include_true ? n_true : 0));
-    SEXP false_locs = Rf_protect(Rf_allocVector(INTSXP, include_false ? n_false : 0));
-    SEXP na_locs = Rf_protect(Rf_allocVector(INTSXP, include_na ? (n - n_true - n_false) : 0));
+    SEXP true_locs = SHIELD(new_vec(INTSXP, include_true ? n_true : 0));
+    SEXP false_locs = SHIELD(new_vec(INTSXP, include_false ? n_false : 0));
+    SEXP na_locs = SHIELD(new_vec(INTSXP, include_na ? (n - n_true - n_false) : 0));
 
     int *p_true = INTEGER(true_locs);
     int *p_false = INTEGER(false_locs);
@@ -621,18 +616,18 @@ SEXP cpp_lgl_locs(SEXP x, R_xlen_t n_true, R_xlen_t n_false,
         p_na[k3++] = i + 1;
       }
     }
-    SEXP out = Rf_protect(Rf_allocVector(VECSXP, 3));
+    SEXP out = SHIELD(new_vec(VECSXP, 3));
     SET_VECTOR_ELT(out, 0, true_locs);
     SET_VECTOR_ELT(out, 1, false_locs);
     SET_VECTOR_ELT(out, 2, na_locs);
 
-    SEXP names = Rf_protect(Rf_allocVector(STRSXP, 3));
-    SET_STRING_ELT(names, 0, Rf_mkChar("true"));
-    SET_STRING_ELT(names, 1, Rf_mkChar("false"));
-    SET_STRING_ELT(names, 2, Rf_mkChar("na"));
-    Rf_setAttrib(out, R_NamesSymbol, names);
+    SEXP names = SHIELD(new_vec(STRSXP, 3));
+    SET_STRING_ELT(names, 0, make_utf8_char("true"));
+    SET_STRING_ELT(names, 1, make_utf8_char("false"));
+    SET_STRING_ELT(names, 2, make_utf8_char("na"));
+    set_names(out, names);
 
-    Rf_unprotect(5);
+    YIELD(5);
     return out;
   }
 }
@@ -645,27 +640,27 @@ SEXP cpp_val_find(SEXP x, SEXP value, bool invert, SEXP n_values){
   if (Rf_length(value) != 1){
     Rf_error("value must be a vector of length 1");
   }
-  if (Rf_isVectorList(x)){
+  if (TYPEOF(x) == VECSXP){
     Rf_error("%s cannot handle an object of type %s", __func__, Rf_type2char(TYPEOF(x)));
   }
 
   bool val_is_na = cpp_any_na(value, true);
 
   if (implicit_na_coercion(value, x)){
-    Rf_unprotect(NP);
+    YIELD(NP);
     Rf_error("Value has been implicitly converted to NA, please check");
   }
 
-  R_xlen_t n_vals = Rf_isNull(n_values) ? scalar_count(x, value, false) : Rf_asReal(n_values);
+  R_xlen_t n_vals = is_null(n_values) ? scalar_count(x, value, false) : Rf_asReal(n_values);
   R_xlen_t out_size = invert ? n - n_vals : n_vals;
   R_xlen_t whichi = 0;
   R_xlen_t i = 0;
   switch ( CHEAPR_TYPEOF(x) ){
   case LGLSXP:
   case INTSXP: {
-    SEXP out = Rf_protect(Rf_allocVector(is_long ? REALSXP : INTSXP, out_size));
+    SEXP out = SHIELD(new_vec(is_long ? REALSXP : INTSXP, out_size));
     ++NP;
-    Rf_protect(value = coerce_vector(value, CHEAPR_TYPEOF(x))); ++NP;
+    SHIELD(value = coerce_vector(value, CHEAPR_TYPEOF(x))); ++NP;
     int val = Rf_asInteger(value);
     int *p_x = INTEGER(x);
     if (is_long){
@@ -683,13 +678,13 @@ SEXP cpp_val_find(SEXP x, SEXP value, bool invert, SEXP n_values){
         CHEAPR_WHICH_VAL(val);
       }
     }
-    Rf_unprotect(NP);
+    YIELD(NP);
     return out;
   }
   case REALSXP: {
-    SEXP out = Rf_protect(Rf_allocVector(is_long ? REALSXP : INTSXP, out_size));
+    SEXP out = SHIELD(new_vec(is_long ? REALSXP : INTSXP, out_size));
     ++NP;
-    Rf_protect(value = coerce_vector(value, REALSXP)); ++NP;
+    SHIELD(value = coerce_vector(value, REALSXP)); ++NP;
     double val = Rf_asReal(value);
     double *p_x = REAL(x);
     if (val_is_na){
@@ -698,13 +693,13 @@ SEXP cpp_val_find(SEXP x, SEXP value, bool invert, SEXP n_values){
         if (invert){
           while (whichi < out_size){
             p_out[whichi] = i + 1;
-            whichi += !cheapr_is_na_dbl(p_x[i]);
+            whichi += !is_na_dbl(p_x[i]);
             ++i;
           }
         } else {
           while (whichi < out_size){
             p_out[whichi] = i + 1;
-            whichi += cheapr_is_na_dbl(p_x[i]);
+            whichi += is_na_dbl(p_x[i]);
             ++i;
           }
         }
@@ -733,15 +728,15 @@ SEXP cpp_val_find(SEXP x, SEXP value, bool invert, SEXP n_values){
         }
       }
     }
-    Rf_unprotect(NP);
+    YIELD(NP);
     return out;
   }
   case CHEAPR_INT64SXP: {
-    SEXP out = Rf_protect(Rf_allocVector(is_long ? REALSXP : INTSXP, out_size));
+    SEXP out = SHIELD(new_vec(is_long ? REALSXP : INTSXP, out_size));
     ++NP;
-    Rf_protect(value = coerce_vector(value, CHEAPR_INT64SXP)); ++NP;
-    long long int val = INTEGER64_PTR(value)[0];
-    long long int *p_x = INTEGER64_PTR(x);
+    SHIELD(value = coerce_vector(value, CHEAPR_INT64SXP)); ++NP;
+    int_fast64_t val = INTEGER64_PTR(value)[0];
+    int_fast64_t *p_x = INTEGER64_PTR(x);
     if (is_long){
       double *p_out = REAL(out);
       if (invert){
@@ -757,14 +752,14 @@ SEXP cpp_val_find(SEXP x, SEXP value, bool invert, SEXP n_values){
         CHEAPR_WHICH_VAL(val);
       }
     }
-    Rf_unprotect(NP);
+    YIELD(NP);
     return out;
   }
   case STRSXP: {
-    SEXP out = Rf_protect(Rf_allocVector(is_long ? REALSXP : INTSXP, out_size));
+    SEXP out = SHIELD(new_vec(is_long ? REALSXP : INTSXP, out_size));
     ++NP;
-    Rf_protect(value = coerce_vector(value, STRSXP)); ++NP;
-    SEXP val = Rf_protect(Rf_asChar(value)); ++NP;
+    SHIELD(value = coerce_vector(value, STRSXP)); ++NP;
+    SEXP val = SHIELD(Rf_asChar(value)); ++NP;
     const SEXP *p_x = STRING_PTR_RO(x);
     if (is_long){
       double *p_out = REAL(out);
@@ -781,19 +776,64 @@ SEXP cpp_val_find(SEXP x, SEXP value, bool invert, SEXP n_values){
         CHEAPR_WHICH_VAL(val);
       }
     }
-    Rf_unprotect(NP);
+    YIELD(NP);
     return out;
   }
   default: {
-    Rf_unprotect(NP);
+    YIELD(NP);
     Rf_error("%s cannot handle an object of type %s", __func__, Rf_type2char(TYPEOF(x)));
   }
   }
 }
 
 
-// 2 more which() alternatives
-// list cpp_which2(SEXP x){
+// 4 alternatives
+
+// Alternative 1 - Using standard R vectors
+// allocate n elements and then shorten
+
+// SEXP cpp_which1(SEXP x){
+//   int n = Rf_length(x);
+//   const int* __restrict__ p_x = LOGICAL(x);
+//   SEXP out = SHIELD(new_vec(INTSXP, n));
+//   int* __restrict__ p_out = INTEGER(out);
+//   int k = 0;
+//   for (int i = 0; i < n; ++i){
+//     if (p_x[i] == TRUE) p_out[k++] = i + 1;
+//   }
+//   SHIELD(out = Rf_lengthgets(out, k));
+//   YIELD(2);
+//   return out;
+// }
+
+// Allocate n/2 elements and push/pop as necessary
+
+// cpp11::integers cpp_which2(cpp11::logicals x){
+//   int n = x.size();
+//   const int* __restrict__ p_x = LOGICAL(x);
+//   int m = n / 2;
+//   cpp11::writable::integers out(m);
+//   int *p_out = INTEGER(out);
+//   int k = 0, l = 0;
+//   // k keeps track of how many elements we've added to out
+//   // l keeps track of how many elements we've removed
+//   for (int i = 0; i < n; ++i){
+//     if (p_x[i] == TRUE){
+//       if (k >= (m - l)){
+//         out.push_back(i + 1);
+//       } else {
+//         p_out[k] = i + 1;
+//       }
+//       ++k;
+//     } else if (k < (m - l)){
+//       out.pop_back();
+//       ++l;
+//     }
+//   }
+//   return out;
+// }
+
+// list cpp_which3(SEXP x){
 //   int n = Rf_xlength(x);
 //   int *p_x = LOGICAL(x);
 //   // std::vector<int> out;
@@ -817,13 +857,13 @@ SEXP cpp_val_find(SEXP x, SEXP value, bool invert, SEXP n_values){
 //   });
 // }
 //
-// SEXP cpp_which3(SEXP x){
+// SEXP cpp_which4(SEXP x){
 //   int n = Rf_xlength(x);
 //   int *p_x = LOGICAL(x);
 //   int size = 0;
 //   int j;
 //   for (j = 0; j < n; ++j) size += (p_x[j] == TRUE);
-//   SEXP out = Rf_protect(Rf_allocVector(INTSXP, size));
+//   SEXP out = SHIELD(new_vec(INTSXP, size));
 //   int *p_out = INTEGER(out);
 //   int k = 0;
 //   for (int i = 0; i < j; ++i){
@@ -831,6 +871,6 @@ SEXP cpp_val_find(SEXP x, SEXP value, bool invert, SEXP n_values){
 //       p_out[k++] = i + 1;
 //     }
 //   }
-//   Rf_unprotect(1);
+//   YIELD(1);
 //   return out;
 // }
